from interfaces import IKSSPlonePortletCommands
from plone.portlets.interfaces import IPortletManager
from plone.app.portlets.portlets.classic import Assignment
from plone.portlets.interfaces import IPortletRenderer
from zope import component

from kss.core.azaxview import AzaxViewAdapter
from zope.interface import implements

class KSSPortletCommands(AzaxViewAdapter):
    implements(IKSSPlonePortletCommands)

    def reload_classic_portlet(self, css_selector, column,
                               template, portlet_macro='portlet'):
        context = self.context
        request = self.request
        view = self.view
        manager = component.getUtility(
            IPortletManager, name=column, context=context)
        assignment = Assignment(
            template=template, macro=portlet_macro)

        portlet = component.getMultiAdapter(
            (context, request, view, manager, assignment), IPortletRenderer)
        portlet = portlet.__of__(context)

        view.getCommandSet('core').replaceInnerHTML(css_selector, portlet.render())



import unittest
from Products.PloneTestCase import PloneTestCase

import plone
from plone.app.kss.azaxview import AzaxBaseView
from plone.app.kss.interfaces import IPortalObject
from plone.app.kss.portlets import attributesTriggerNavigationPortletReload
from plone.app.kss.portlets import attributesTriggerRecentPortletReload
from plone.app.kss.tests.kss_and_plone_layer import KSSAndPloneTestCase

from zope.lifecycleevent import ObjectModifiedEvent
from zope import lifecycleevent

from Products.Archetypes.event import ObjectEditedEvent
from Products.Five.zcml import load_config

PloneTestCase.setupPloneSite()

class SampleView(AzaxBaseView):

    def change_title(self, title):
        # normally you would change the zope database here
        self.handle(ObjectModifiedEvent(self.context))
        return self.render()


class TestPortletReloading(KSSAndPloneTestCase):
    class layer(KSSAndPloneTestCase.layer):
        @classmethod
        def setUp(cls):
            load_config('configure-part_reloading.zcml',
                        package=plone.app.kss.tests)            

    def afterSetUp(self):
        PloneTestCase.PloneTestCase.afterSetUp(self)
        self.setDebugRequest()
        self.loginAsPortalOwner()
        self.setRoles(['Manager',])
        # register the sample view

        self.view = self.portal.restrictedTraverse('@@change_title')

    def test_no_update_of_nav_portlet_when_unhooked(self):
        # nothing should happen
        result = self.view.render()
        self.assertEqual(result, [])

    def test_no_update_of_nav_portlet_when_hooked_with_wrong_event(self):
        # nothing should happen still because we must change the title or the
        # description
        modified_event = ObjectEditedEvent(self.folder)
        attributesTriggerNavigationPortletReload(self.folder, self.view, modified_event)
        result = self.view.render()
        self.assertEqual(result, [])

    def test_update_of_nav_portlet(self):
        descriptor = lifecycleevent.Attributes(IPortalObject, 'title')
        modified_event = ObjectEditedEvent(self.folder, descriptor)
        attributesTriggerNavigationPortletReload(self.folder, self.view, modified_event)
        result = self.view.render()
        command = result[0]
        self.failUnless(command.has_key('selector'))
        self.failUnless(command['selector'].startswith('portletwrapper'))
        self.failUnless(command.has_key('name'))
        self.assertEqual(command['name'], 'replaceInnerHTML')
        self.failUnless(command.has_key('params'))
        params = result[0]['params']
        self.failUnless(params.has_key('html'))
        html = params['html']
        self.failUnless('portletNavigationTree' in html)
        self.failUnless(command.has_key('selectorType'))
        self.assertEqual(command['selectorType'], 'htmlid')

    def test_update_of_recent_portlet(self):
        descriptor = lifecycleevent.Attributes(IPortalObject, 'title')
        modified_event = ObjectEditedEvent(self.folder, descriptor)
        attributesTriggerRecentPortletReload(self.folder, self.view, modified_event)
        result = self.view.render()
        command = result[0]
        self.failUnless(command.has_key('selector'))
        self.failUnless(command['selector'].startswith('portletwrapper'))
        self.failUnless(command.has_key('name'))
        self.assertEqual(command['name'], 'replaceInnerHTML')
        self.failUnless(command.has_key('params'))
        params = result[0]['params']
        self.failUnless(params.has_key('html'))
        html = params['html']
        self.failUnless('portletRecent' in html)
        self.failUnless(command.has_key('selectorType'))
        self.assertEqual(command['selectorType'], 'htmlid')

def test_suite():
    suites = []
    suites.append(unittest.makeSuite(TestPortletReloading))
    return unittest.TestSuite(suites)



from zope.interface import Interface

class IIssuePortalMessageCommand(Interface):
    '''effects commands'''

class IRefreshPortletCommand(Interface):
    '''effects commands'''

class IKSSPlonePortletCommands(Interface):
    '''These are utility commands for doing stuff with portlets'''

    def reload_classic_portlet(css_selector, column,
                               template, portlet_macro='portlet'):
        '''Reload an old-school portlet'''

class IRefreshProviderCommand(Interface):
    '''effects commands'''
    
    def refreshProvider(self, name, selector):
	    '''refreshes a generic IContentProvider named <name> and
        located (in the HTML) at <selector> (css selector)'''

class IReplaceContentMenuCommand(Interface):
    '''effects commands'''
    
    def replaceMenu(self):
	    '''replace the content menu'''



from kss.core.azaxview import AzaxViewAdapter

class ReplaceContentMenuCommand(AzaxViewAdapter):
    """Mainly exists for backward compatibility with old hooks
    """
    def replaceMenu(self):
        self.getCommandSet('refreshprovider').refreshProvider('plone.contentmenu', '#portal-column-content div.contentActions')



import Acquisition
from zope.component.interfaces import ComponentLookupError
import zope.component.persistentregistry
import five.localsitemanager.registry
import OFS.ObjectManager
import logging
logger = logging.getLogger('plone.app.kss')

_marker = object()

# XXX monkeypatching zope
# this will make sure, if our utility provides the marker, we will
# provide the necessary parameters to it
class PatchComponent(object):

    def queryUtility(self, provided, name=u'', default=None):
        utility = self.utilities.lookup((), provided, name, default)
        if utility is not default:
            utility = self._wrap(utility)
        return utility

    def getUtility(self, provided, name=u''):
        utility = self.queryUtility(provided, name=name, default=_marker)
        if utility is _marker:
            raise ComponentLookupError(provided, name)
        return utility

    def getUtilitiesFor(self, interface):
        for name, utility in self.utilities.lookupAll((), interface):
            utility = self._wrap(utility)
            yield name, utility

    def _wrap(self, comp):
        """Return an aq wrapped component with the site as the parent but
        only if the comp has an aq wrapper to begin with.
        """

        # BBB: The primary reason for doing this sort of wrapping of
        # returned utilities is to support CMF tool-like functionality where
        # a tool expects it's aq_parent to be the portal object.  New code
        # (ie new utilities) should not rely on this predictability to
        # get the portal object and should search out an alternate means
        # (possibly retrieve the ISiteRoot utility).  Although in most
        # cases getting at the portal object shouldn't be the required pattern
        # but instead looking up required functionality via other (possibly
        # local) components.

        if Acquisition.interfaces.IAcquirer.providedBy(comp): 

            # sitemanager contains the sitemamager that the caller called
            # the query on. Howewer this is not necessarily the same sitemanager
            # on which this regisrty is on. We need to iterate on all site managers
            # and figure out which ones we are.
            sitemanager = self._select_sm()
            if sitemanager is None:
                # no sm provides acquisition
                return comp

            parent = sitemanager.aq_parent
            base = Acquisition.aq_base(comp)

            if base is not Acquisition.aq_base(parent):
                # If the component is not the component registry container,
                # wrap it in the parent
                comp = base.__of__(parent)
            else:
                # If the component happens to be the component registry
                # container we are looking up a ISiteRoot.
                # We are not wrapping it in itself but in its own parent
                comp = base.__of__(Acquisition.aq_parent(parent))

        return comp

    def _iter_all_sitemanagers(self, sitemanager):
        yield sitemanager
        for chained_sm in sitemanager.__bases__:
            for more in self._iter_all_sitemanagers(chained_sm):
                yield more

    def _select_sm(self):
        '''Select the correct site manager

        choose one from the site managers, that have acquisition info

        We should actually return the sitemanager that has the registry
        But unfortunately, we cannot know _which_ sitemanager is giving us
        the registry, so the first one that has aq wrapping, will be used
        XXX and this is totally wrong in fact.
        '''
        for sm in self._iter_all_sitemanagers(self):
            if Acquisition.aq_parent(sm) is not None:
                break
        else:
            # Site manager not found (no aq info, sorry)
            sm = None
        return sm

# apply the monkeypatches
import zope.component.registry
for key in PatchComponent.__dict__.iterkeys():
    if not key.startswith('__'):
        setattr(zope.component.registry.Components, key,
                getattr(PatchComponent, key).im_func)
logger.info('*** monkeypatching zope.component.registry.Components ***')

# XXX Now we need to patch five.localsitamanager too.
# Because the acquisition is originally added from there, but since we have relocated
# this functionality to the z3 level, it will be applied for every site manager, so no
# reason to apply it also from Five's one.

def _init_registries(self):
    self.adapters = zope.component.persistentregistry.PersistentAdapterRegistry()
    self.utilities = zope.component.persistentregistry.PersistentAdapterRegistry()

for key in ('queryUtility', 'getUtility', 'getUtilitiesFor',
        'getAllUtilitiesRegisteredFor'):
    setattr(five.localsitemanager.registry.PersistentComponents, key,
            getattr(zope.component.persistentregistry.PersistentComponents, key))
five.localsitemanager.registry.PersistentComponents._init_registries = \
        _init_registries
logger.info('*** monkeypatching five.localsitemanager.registry.PersistentComponents ***')



from Products.PloneTestCase.layer import PloneSite
from Products.PloneTestCase import PloneTestCase as ptc
from kss.core.tests.base import KSSLayer, KssViewTestCase

class KSSAndPloneLayer(PloneSite, KSSLayer):
    pass

class KSSAndPloneTestCase(ptc.PloneTestCase, KssViewTestCase):
    layer = KSSAndPloneLayer
